﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using MFORMATSLib;

namespace TutorialLiveInput
{
    public partial class Form1 : Form
    {
        struct MousePos
        {
            public double dblX;
            public double dblY;
        }

        private MFPreviewClass m_objPreview;
        private IMFBrowser m_objOverlayHTML;
        private MFLiveClass m_objLive;
        private Thread m_threadWorker;	//Working thread
        private bool m_bWork;
        private M_VID_PROPS m_vProps;

        public Form1()
        {
            InitializeComponent();
        }

        private CancellationTokenSource cancelSource;
        private void Form1_Load(object sender, EventArgs e)
        {
            m_objPreview = new MFPreviewClass();
            try
            {
                m_objLive = new MFLiveClass();
            }
            catch (Exception ex)
            {
                MessageBox.Show("MLive device not found\n\n" + ex.ToString());
            }
            m_objPreview.PropsSet("preview.downscale", "0");
            m_objPreview.PropsSet("dx11.windowless", "true");
            m_objPreview.PropsSet("deinterlace", "false");
            //Configure preview
            m_objPreview.PreviewWindowSet("", panelPreview.Handle.ToInt32());
            m_objPreview.PreviewEnable("", 1, 1);
            panelPreview.MouseWheel += new MouseEventHandler(panelPreview_MouseWheel);
            panelPreview.KeyDown += new KeyEventHandler(panelPreview_KeyDown);
            panelPreview.KeyPress += new KeyPressEventHandler(panelPreview_KeyPress);
            panelPreview.KeyUp += new KeyEventHandler(panelPreview_KeyUp);

            //Fill live devices
            FillMLiveDevices();

            // Create & config background worker
            cancelSource = new CancellationTokenSource();
            m_threadWorker = new Thread(() => thread_DoWork(cancelSource.Token));
            m_threadWorker.Name = "thread_DoWork";
			m_threadWorker.Start();

            System.Windows.Forms.Control.CheckForIllegalCrossThreadCalls = false;   //Need for use renderersCombobox in multithreading 
        }

        void panelPreview_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Back || e.KeyCode == Keys.ShiftKey || e.KeyCode == Keys.ControlKey || e.KeyCode == Keys.Alt ||
                e.KeyCode == Keys.Left || e.KeyCode == Keys.Right || e.KeyCode == Keys.Up || e.KeyCode == Keys.Down)
            {
                m_objOverlayHTML.BrowserKeyEvent(GetKey(Convert.ToInt32(e.KeyCode), eMFKeyEventType.eMFKT_KeyDown));
            }
        }

        private void panelPreview_MouseDown(object sender, MouseEventArgs e)
        {
            panelPreview.Focus();

            MF_MOUSE_EVENT mouseEvent = new MF_MOUSE_EVENT();
            mouseEvent.dblPosX = GetPos((double)e.X, (double)e.Y).dblX;
            mouseEvent.dblPosY = GetPos((double)e.X, (double)e.Y).dblY;

            mouseEvent.eFlags = MouseButtons2eMFFlag(e.Button);
            if (m_objOverlayHTML!=null)
                m_objOverlayHTML.BrowserMouseClick(mouseEvent, MouseButtons2eMF(e.Button), 0, 1);
        }
        void panelPreview_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (m_objOverlayHTML != null)
                m_objOverlayHTML.BrowserKeyEvent(GetKey(Convert.ToInt32(e.KeyChar), eMFKeyEventType.eMFKT_Char));
        }
        private MF_KEY_EVENT GetKey(int _nKeyCode, eMFKeyEventType _keyEventType)
        {
            MF_KEY_EVENT keyEvent = new MF_KEY_EVENT();
            keyEvent.eType = _keyEventType;
            keyEvent.windows_key_code = _nKeyCode;

            return keyEvent;
        }

        void panelPreview_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Back || e.KeyCode == Keys.ShiftKey || e.KeyCode == Keys.ControlKey || e.KeyCode == Keys.Alt ||
                e.KeyCode == Keys.Left || e.KeyCode == Keys.Right || e.KeyCode == Keys.Up || e.KeyCode == Keys.Down)
            {
                if (m_objOverlayHTML != null)
                    m_objOverlayHTML.BrowserKeyEvent(GetKey(Convert.ToInt32(e.KeyCode), eMFKeyEventType.eMFKT_KeyUp));
            }
        }
        private void panelPreview_MouseUp(object sender, MouseEventArgs e)
        {
            MF_MOUSE_EVENT mouseEvent = new MF_MOUSE_EVENT();
            mouseEvent.dblPosX = GetPos((double)e.X, (double)e.Y).dblX;
            mouseEvent.dblPosY = GetPos((double)e.X, (double)e.Y).dblY;

            mouseEvent.eFlags = MouseButtons2eMFFlag(e.Button);
            if (m_objOverlayHTML != null)
                m_objOverlayHTML.BrowserMouseClick(mouseEvent, MouseButtons2eMF(e.Button), 1, 1);
        }

        double dblPrevX = 0;
        double dblPrevY = 0;
        private void panelPreview_MouseMove(object sender, MouseEventArgs e)
        {
            MF_MOUSE_EVENT mouseEvent = new MF_MOUSE_EVENT();
            mouseEvent.dblPosX = GetPos((double)e.X, (double)e.Y).dblX;
            mouseEvent.dblPosY = GetPos((double)e.X, (double)e.Y).dblY;

            if (mouseEvent.dblPosX == dblPrevX && mouseEvent.dblPosY == dblPrevY)
                return;

            mouseEvent.eFlags = MouseButtons2eMFFlag(e.Button);
            if (m_objOverlayHTML != null)
                m_objOverlayHTML.BrowserMouseMove(mouseEvent, 0);

            dblPrevX = mouseEvent.dblPosX;
            dblPrevY = mouseEvent.dblPosY;
        }
        private eMFEventsFlags MouseButtons2eMFFlag(MouseButtons _mouseButton)
        {
            eMFEventsFlags eflag = 0;

            if (_mouseButton.HasFlag(MouseButtons.Left))
                eflag = eMFEventsFlags.eMFEF_LEFT_MOUSE_BUTTON;
            if (_mouseButton.HasFlag(MouseButtons.Right))
                eflag |= eMFEventsFlags.eMFEF_RIGHT_MOUSE_BUTTON;
            if (_mouseButton.HasFlag(MouseButtons.Middle))
                eflag |= eMFEventsFlags.eMFEF_MIDDLE_MOUSE_BUTTON;

            return eflag;
        }
        private void panelPreview_MouseWheel(object sender, MouseEventArgs e)
        {
            MF_MOUSE_EVENT mouseEvent = new MF_MOUSE_EVENT();
            mouseEvent.dblPosX = GetPos((double)e.X, (double)e.Y).dblX;
            mouseEvent.dblPosY = GetPos((double)e.X, (double)e.Y).dblY;
            if(m_objOverlayHTML != null)
                m_objOverlayHTML.BrowserMouseWheel(mouseEvent, 0, e.Delta);
        }

        private MousePos GetPos(double _x, double _y)
        {
            MousePos mousePosReturn;

            //if (!checkBoxAR.Checked)
            //{
            //    mousePosReturn.dblX = _x / panelPreview.Width;
            //    mousePosReturn.dblY = _y / panelPreview.Height;

            //    return mousePosReturn;
            //}

            if ((double)m_vProps.nWidth / m_vProps.nHeight > (double)panelPreview.Width / panelPreview.Height)
            {
                mousePosReturn.dblX = _x / panelPreview.Width;

                double dblHeighInt = (double)panelPreview.Width * m_vProps.nHeight / m_vProps.nWidth;
                double nYInt = (panelPreview.Height - dblHeighInt) / 2;
                if (_y < nYInt || _y > panelPreview.Height - nYInt)
                {
                    mousePosReturn.dblY = 0;
                }
                else
                {
                    mousePosReturn.dblY = (_y - nYInt) / dblHeighInt;
                }
            }
            else
            {
                mousePosReturn.dblY = _y / panelPreview.Height;

                double dblWidthInt = (double)panelPreview.Height * m_vProps.nWidth / m_vProps.nHeight;
                double nXInt = (panelPreview.Width - dblWidthInt) / 2;
                if (_x < nXInt || _x > panelPreview.Width - nXInt)
                {
                    mousePosReturn.dblX = 0;
                }
                else
                {
                    mousePosReturn.dblX = (_x - nXInt) / dblWidthInt;
                }
            }

            return mousePosReturn;
        }

        private eMFMouseButtons MouseButtons2eMF(MouseButtons _mouseButton)
        {
            switch (_mouseButton)
            {
                case MouseButtons.None: return eMFMouseButtons.eMFMB_None;
                case MouseButtons.Left: return eMFMouseButtons.eMFMB_Left;
                case MouseButtons.Right: return eMFMouseButtons.eMFMB_Right;
                case MouseButtons.Middle: return eMFMouseButtons.eMFMB_Middle;

                default: return eMFMouseButtons.eMFMB_None;
            }
        }

        private void thread_DoWork(CancellationToken token)
        {
            while (!token.IsCancellationRequested)
            {
                if (m_bWork)
                {
                    MFFrame pFrame = null;

                    try
                    {
                        m_objLive.SourceFrameGet(-1, out pFrame, "");
                        M_AV_PROPS props;
                        int samples;
                        pFrame.MFAVPropsGet(out props, out samples);
                        //Send frame to the preview
                        m_vProps = props.vidProps;
                        m_objPreview.ReceiverFramePut(pFrame, -1, "");

                        //Release frame - DO NOT FORGOT TO DO THIS !!!
                        Marshal.ReleaseComObject(pFrame);
                    }
                    catch
                    {
                    }
                }
                else
                {
                    Thread.Sleep(1);
                }
            }
        }


        private void FillMLiveDevices()
        {
            comboBoxVideo.Items.Clear();
            //Fill video devices
            int vCount;
            try
            {
                m_objLive = new MFLiveClass();

                m_objLive.DeviceGetCount(eMFDeviceType.eMFDT_Video, out vCount);

                if (vCount > 0)
                {
                    for (int i = 0; i < vCount; i++)
                    {
                        string strName;
                        int _pbBusy;
                        m_objLive.DeviceGetByIndex(eMFDeviceType.eMFDT_Video, i, out strName, out _pbBusy);
                        comboBoxVideo.Items.Add(strName);

                    }
                    comboBoxVideo.SelectedIndex = 0;
                    
                    buttonInit.Enabled = true;
                    buttonClose.Enabled = true;
                }

            }
            catch { }
        }

        private void comboBoxV_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                comboBoxFormat.Items.Clear();
                comboBoxVideoLine.Items.Clear();

                M_VID_PROPS vidProps;

                m_objLive.DeviceSet(eMFDeviceType.eMFDT_Video, comboBoxVideo.SelectedIndex, "");

                buttonInit.Enabled = true;
                buttonClose.Enabled = true;

                int fCount, lCount;
                string name, _pOptions, _pHelp;

                m_objLive.FormatVideoGetCount(eMFormatType.eMFT_Input, out fCount);
                if (fCount > 0)
                {
                    for (int i = 0; i < fCount; i++)
                    {
                        m_objLive.FormatVideoGetByIndex(eMFormatType.eMFT_Input, i, out vidProps, out name);
                        comboBoxFormat.Items.Add(name);
                    }

                    // Get current format
                    int nCurrent = 0;
                    try
                    {
                        string strName;
                        M_VID_PROPS _vidProps;
                        m_objLive.FormatVideoGet(eMFormatType.eMFT_Input, out _vidProps, out nCurrent, out strName);
                    }
                    catch (System.Exception)
                    {
                    }

                    // Select current
                    comboBoxFormat.SelectedIndex = nCurrent > 0 ? nCurrent : 0;
                }

                string strCurrent;
                int nSel = -1;
                m_objLive.PropsGet("line-in", out strCurrent);

                m_objLive.PropsOptionGetCount("line-in", out lCount);
                if (lCount > 0)
                {
                    for (int i = 0; i < lCount; i++)
                    {
                        m_objLive.PropsOptionGetByIndex("line-in", i, out _pOptions, out _pHelp);
                        comboBoxVideoLine.Items.Add(_pHelp);

                        if (strCurrent == _pOptions)
                            nSel = i;
                    }

                    if (nSel >= 0)
                        comboBoxVideoLine.SelectedIndex = nSel;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void comboBoxVL_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_objLive.PropsOptionSetByIndex("line-in", comboBoxVideoLine.SelectedIndex);
            if (comboBoxVideo.SelectedItem.ToString().Contains("WebCapture"))
            {
                object internalObj;
                m_objLive.DeviceGetInternal(eMFDeviceType.eMFDT_Video, out internalObj);
                if (internalObj is IMFBrowser)
                    m_objOverlayHTML = internalObj as IMFBrowser;

            }
        }

        private void comboBoxVF_SelectedIndexChanged(object sender, EventArgs e)
        {
            M_VID_PROPS vidProps;
            string name;
            m_objLive.FormatVideoGetByIndex(eMFormatType.eMFT_Input, comboBoxFormat.SelectedIndex, out vidProps, out name);

            m_objLive.FormatVideoSet(eMFormatType.eMFT_Input, vidProps);
        }

        private void btnInit_Click(object sender, EventArgs e)
        {
            // Set device
            m_objLive.DeviceSet(eMFDeviceType.eMFDT_Video, comboBoxVideo.SelectedIndex, "");

            // Set format
            M_VID_PROPS vidProps;
            string name;
            m_objLive.FormatVideoGetByIndex(eMFormatType.eMFT_Input, comboBoxFormat.SelectedIndex, out vidProps, out name);
            m_objLive.FormatVideoSet(eMFormatType.eMFT_Input, vidProps);

            if (comboBoxVideo.SelectedItem.ToString().Contains("WebCapture"))
            {
                object internalObj;
                m_objLive.DeviceGetInternal(eMFDeviceType.eMFDT_Video, out internalObj);
                if (internalObj is IMFBrowser)
                    m_objOverlayHTML = internalObj as IMFBrowser;

            }
            // Start worker
            m_bWork = true;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            m_bWork = false;
            m_objLive.DeviceClose();
        }

        private void panelPreview_SizeChanged(object sender, EventArgs e)
        {
			if (m_objPreview != null )
				m_objPreview.PreviewWindowSet("", panelPreview.Handle.ToInt32());
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            cancelSource.Cancel();
            m_threadWorker.Join(1000);
			
            m_objLive.DeviceClose();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            OpenFileDialog oDialog = new OpenFileDialog();
            oDialog.Filter = "PDF files(*.pdf)|*.pdf";
            if (oDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    m_objLive.PropsSet("line-in", oDialog.FileName);
                    if (comboBoxVideo.SelectedItem.ToString().Contains("WebCapture"))
                    {
                        object internalObj;
                        m_objLive.DeviceGetInternal(eMFDeviceType.eMFDT_Video, out internalObj);
                        if (internalObj is IMFBrowser)
                            m_objOverlayHTML = internalObj as IMFBrowser;

                    }
                }
                catch { }
            }
        }
    }
}
